﻿using Kivii.Test.WebDriver.Tests;
using Kivii.WebDriver;
using Kivii.WebDriver.Actions;
using Newtonsoft.Json;
using OpenQA.Selenium.Firefox;
using OpenQA.Selenium.Remote;
using OpenQA.Selenium.Support.Events;
using System;
using System.Linq;
using System.Windows.Forms;

namespace Kivii.Test.WebDriver
{
    public partial class FrmMain : Form
    {
        /// <summary>
        /// 管理器
        /// </summary>
        private Manager _manager=null;

        #region 初始化
        public FrmMain()
        {
            InitializeComponent();
        }
        #endregion

        #region 输出日志相关
        private void appendMsg(string msg)
        {
            rtbMsg.AppendText($"{msg}{Environment.NewLine}");
            rtbMsg.ScrollToCaret();
        }
        /// <summary>
        /// 双击输出消息，清除内容
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rtbMsg_DoubleClick(object sender, EventArgs e)
        {
            rtbMsg.Clear();
        }
        #endregion

        #region 监听管理器的事件
        private void addListener()
        {
            if (_manager == null) return;

            _manager.OnException += manager_OnException;
            _manager.OnCurrentWindowChanged += manager_OnWindowChanged;
            _manager.OnActionExecuted += manager_OnActionExecuted;
            _manager.OnActionException += manager_OnActionException;
            //以下是WebDriver的事件
            _manager.ElementClicked += EventDriver_ElementClicked;
            _manager.ElementClicking += EventDriver_ElementClicking;
            _manager.ElementValueChanging += EventDriver_ElementValueChanging;
            _manager.ElementValueChanged += EventDriver_ElementValueChanged;
            _manager.ExceptionThrown += EventDriver_ExceptionThrown;
            _manager.FindElementCompleted += EventDriver_FindElementCompleted;
            _manager.FindingElement += EventDriver_FindingElement;
            _manager.GetShadowRootCompleted += EventDriver_GetShadowRootCompleted;
            _manager.GettingShadowRoot += EventDriver_GettingShadowRoot;
            _manager.Navigated += EventDriver_Navigated;
            _manager.NavigatedBack += EventDriver_NavigatedBack;
            _manager.NavigatedForward += EventDriver_NavigatedForward;
            _manager.Navigating += EventDriver_Navigating;
            _manager.NavigatingBack += EventDriver_NavigatingBack;
            _manager.NavigatingForward += EventDriver_NavigatingForward;
            _manager.ScriptExecuted += EventDriver_ScriptExecuted;
            _manager.ScriptExecuting += EventDriver_ScriptExecuting;
        }

        private void manager_OnActionException(Window sender, Exception e)
        {
            appendMsg($"动作执行异常:{DateTime.Now}{Environment.NewLine}{e.Message}{Environment.NewLine}{e.StackTrace}");
        }

        private void manager_OnActionExecuted(Window sender, string e)
        {
            appendMsg($"动作执行成功:{DateTime.Now}{Environment.NewLine}{e}");
        }

        private void manager_OnWindowChanged(object sender, Window e)
        {
            lblCurrenWindow.Text = $"当前窗口:{(_manager?.StartWindow==_manager.CurrentWindow?"主窗口":"子窗口")}{Environment.NewLine}{e.Handle}{Environment.NewLine}标题:{e.Title}";
        }

        private void manager_OnException(object sender, Exception e)
        {
            appendMsg($"管理器执行异常:{DateTime.Now}{Environment.NewLine}{e.Message}{Environment.NewLine}{e.StackTrace}");
        }

        private void EventDriver_ScriptExecuting(object sender, WebDriverScriptEventArgs e)
        {
            appendMsg($"网页脚本执行:{Environment.NewLine}{e.Script}");
        }

        private void EventDriver_ScriptExecuted(object sender, WebDriverScriptEventArgs e)
        {
            appendMsg($"网页脚本执行成功:{Environment.NewLine}{e.Script}");
        }

        private void EventDriver_NavigatingForward(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatingForward:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_NavigatingBack(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatingBack:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_Navigating(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_Navigating:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_NavigatedForward(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatedForward:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_NavigatedBack(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatedBack:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_Navigated(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_Navigated:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_GettingShadowRoot(object sender, GetShadowRootEventArgs e)
        {
            appendMsg($"EventDriver_GettingShadowRoot:{Environment.NewLine}{e}");
        }

        private void EventDriver_GetShadowRootCompleted(object sender, GetShadowRootEventArgs e)
        {
            appendMsg($"EventDriver_GetShadowRootCompleted:{Environment.NewLine}{e}");
        }

        private void EventDriver_FindingElement(object sender, FindElementEventArgs e)
        {
            appendMsg($"EventDriver_FindingElement:{Environment.NewLine}{e}");
        }

        private void EventDriver_FindElementCompleted(object sender, FindElementEventArgs e)
        {
            appendMsg($"EventDriver_FindElementCompleted:{Environment.NewLine}{e}");
        }

        private void EventDriver_ExceptionThrown(object sender, WebDriverExceptionEventArgs e)
        {
            appendMsg($"EventDriver_ExceptionThrown:{Environment.NewLine}{e.ThrownException?.Message}");
        }

        private void EventDriver_ElementValueChanged(object sender, WebElementValueEventArgs e)
        {
            appendMsg($"EventDriver_ElementValueChanged:{Environment.NewLine}{e}");
        }

        private void EventDriver_ElementValueChanging(object sender, WebElementValueEventArgs e)
        {
            appendMsg($"EventDriver_ElementValueChanging:{Environment.NewLine}{e}");
        }

        private void EventDriver_ElementClicking(object sender, WebElementEventArgs e)
        {
            appendMsg($"EventDriver_ElementClicking:{Environment.NewLine}{e}");
        }

        private void EventDriver_ElementClicked(object sender, WebElementEventArgs e)
        {
            appendMsg($"EventDriver_ElementClicked:{Environment.NewLine}{e}");
        }
        #endregion

        #region 组织管理器的配置
        /// <summary>
        /// 使用远程WebDriver选择框发生变化 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxRemote_CheckedChanged(object sender, EventArgs e)
        {
            lnkVncUrl.Enabled = cbxRemote.Checked;
            cbUseHub.Enabled = cbxRemote.Checked;
            cbxHidenLocalCommandWindow.Visible=!cbxRemote.Checked;
            cbxBrowserType_SelectedIndexChanged(null, null);
        }
        /// <summary>
        /// 浏览器类型选择发生变化 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxBrowserType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!cbxRemote.Checked)
            {
                lnkVncUrl.Text = null;
                return;
            }
            if (cbUseHub.Checked)
            {
                lnkVncUrl.Text = $"http://hub.selenium.kivii.org";
            }
            else
            {
                if (string.IsNullOrEmpty(cbxBrowserType.Text))
                {
                    lnkVncUrl.Text = null;
                }
                else
                {
                    lnkVncUrl.Text = $"http://viewer.{cbxBrowserType.Text}.selenium.kivii.org";
                }
            }
        }
        /// <summary>
        /// 点击了远程浏览器VNC
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lnkVncUrl_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (cbUseHub.Checked)
            {
                System.Diagnostics.Process.Start($"{lnkVncUrl.Text}");
            }
            else
            {
                System.Diagnostics.Process.Start($"{lnkVncUrl.Text}/?autoconnect=1&resize=scale&password=secret");
            }
        }
        #endregion

        #region 初始化和销毁管理器
        /// <summary>
        /// 初始化管理器
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnInitialize_Click(object sender, EventArgs e)
        {
            if (_manager != null) return;
            rtbMsg.Clear();
            _manager = new Manager();
            if (cbBrowserUndetected.Checked) _manager.Options |= BrowserOption.Undetected;//不显示浏览器
            if (cbBrowserHeadless.Checked) _manager.Options |= BrowserOption.Headless;//不显示浏览器
            if (cbBrowserImageless.Checked) _manager.Options |= BrowserOption.Imageless;//不显示浏览器
            if (cbBrowserCssless.Checked) _manager.Options |= BrowserOption.Cssless;//不显示浏览器

            _manager.HideLocalService = cbxHidenLocalCommandWindow.Checked;//如果是本地运行，可选择隐藏本地服务的命令行
            addListener();//管理器的2个事件是直接可以监听的，WebDriver的事件是需要等初始化完才能监听的
            var serviceUrl = string.Empty;
            if (cbxRemote.Checked)
            {
                if (cbUseHub.Checked)
                {
                    serviceUrl = "http://hub.selenium.kivii.org/";
                }
                else
                {
                    serviceUrl = $"http://driver.{cbxBrowserType.Text}.selenium.kivii.org";
                }
            }
            var result=_manager.Initialize(cbxBrowserType.Text, serviceUrl)?"成功":"失败，可能是未能正确加载驱动";
            appendMsg($"初始化{result}");
        }
        /// <summary>
        /// 销毁管理器
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDispose_Click(object sender, EventArgs e)
        {
            _manager?.Dispose();
            _manager = null;
        }
        #endregion

        #region 原生操作
        /// <summary>
        /// 浏览器截图
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnScreenshot_Click(object sender, EventArgs e)
        {
            //if (_driver == null) return;
            SaveFileDialog sfd=new SaveFileDialog();
            sfd.Filter = "图片文件|*.jpg";
            if (sfd.ShowDialog() != DialogResult.OK) return;
            _manager?.CurrentWindow.Screenshot(sfd.FileName);
        }
        /// <summary>
        /// 向网页加载一个脚本路径
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLoadScript_Click(object sender, EventArgs e)
        {
            _manager?.CurrentWindow.LoadScript(tbxScriptPath.Text);
        }

        /// <summary>
        /// 导航至某个网页
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNavigate_Click(object sender, EventArgs e)
        {
            _manager?.CurrentWindow.GoTo(tbxUrl.Text);
        }

        /// <summary>
        /// 显示当前浏览器中所有的窗口信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnHandlers_Click(object sender, EventArgs e)
        {
            foreach (var handler in _manager.AllWindowHandles)
            {
                appendMsg($"Handle Name:{handler}");
            }
            appendMsg($"Current Handle:{_manager.CurrentWindowHandle}");
        }

        /// <summary>
        /// 向执行器加入一段代码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAutoAppendScript_Click(object sender, EventArgs e)
        {
            rtbEngineScript.AppendText($"{cbScriptTip.Text}{Environment.NewLine}");
        }
        /// <summary>
        /// 向管理器引擎注入一个变量
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSetReference_Click(object sender, EventArgs e)
        {
            //向Manager中注入一个变量，可以在引擎执行代码中调用，变量名为tbxVarName中配置的
            //这个功能可以注入一个服务器端存储或读取数量的类，方便交互,
            _manager?.SetReference(tbxVarName.Text, new ReferenceTest());
            appendMsg($"注入了变量名为{tbxVarName.Text},可在Manager.ExecuteScript()的脚本代码中使用该变量");
        }

        /// <summary>
        /// 显示当前应用程序域中所有Action动作的类型
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnShowActionsTip_Click(object sender, EventArgs e)
        {
            foreach (var actionAttribute in ActionBase.AllActions())
            {
                appendMsg($"{actionAttribute.Type.FullName}{Environment.NewLine}别名:\"{actionAttribute.Alias}\"    名称:{actionAttribute.Name}{Environment.NewLine}描述：{actionAttribute.Description}");
            }
        }
        private void cbWindows_Enter(object sender, EventArgs e)
        {
            cbWindows.Items.Clear();
            var windows = _manager?.AllWindowHandles.ToArray();
            if (windows == null) return;
            cbWindows.Items.AddRange(windows);
        }
        /// <summary>
        /// 选择一个窗口激活
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSwitchWindow_Click(object sender, EventArgs e)
        {
            _manager?.SwitchWindow(cbWindows.Text);
        }
        /// <summary>
        /// 关闭当前窗口
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCloseCurrentWindow_Click(object sender, EventArgs e)
        {
            _manager?.CurrentWindow?.Close();
        }
        #endregion



        #region 三个脚本执行功能
        /// <summary>
        /// 执行Manager内置脚本引擎
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnManagerExecuteScript_Click(object sender, EventArgs e)
        {
            var result = _manager.ExecuteScript(rtbEngineScript.Text);
            rtbEngineScript.Clear();
            appendMsg($"Result:{result}");
        }

        /// <summary>
        /// 执行网页内的Javascript
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnWindowExecuteScript_Click(object sender, EventArgs e)
        {
            var result = _manager?.CurrentWindow.ExecuteScript(rtbWebScript.Text);
            rtbWebScript.Clear();
            appendMsg($"Result:{result}");
        }

        /// <summary>
        /// 将Actions的JSON代码加载进WebPage中执行，URL的起始页使用界面上的tbxUrl中的值
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnActionsJson_Click(object sender, EventArgs e)
        {
            if (_manager == null) return;

            _manager.CurrentWindow.ImportActions(rtbActionsJson.Text);
            var rtns = _manager.CurrentWindow.ExecuteActions();
            appendMsg($"执行结果:{Environment.NewLine}{rtns}");

        }

        #endregion

        #region Action的演示代码
        /// <summary>
        /// 加载一段演示动作的Actions代码，JSON格式
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnActionsDemo_Click(object sender, EventArgs e)
        {
            var actions = WeatherTest.GetActions();
            var json = actions.ExportActions(true);
            rtbActionsJson.Text = json;
        }
        /// <summary>
        /// 鼠标动作示例
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMouseDemo_Click(object sender, EventArgs e)
        {
            var autoPage = MouseTest.Get();
            var json = autoPage.ExportActions(true);
            rtbActionsJson.Text = json;
        }
        private void btnMofcomTest_Click(object sender, EventArgs e)
        {
            var autoPage = MofcomTest.Get();
            var json = autoPage.ExportActions(true);
            rtbActionsJson.Text = json;
        }
        /// <summary>
        /// 导入JSON文件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnImportActions_Click(object sender, EventArgs e)
        {
            var ofd = new OpenFileDialog();
            ofd.Filter = "脚本文件|*.json";
            if (ofd.ShowDialog() != DialogResult.OK) return;
            var json = System.IO.File.ReadAllText(ofd.FileName);
            rtbActionsJson.Text = json;
        }
        #endregion
    }
}
